@@ -301,6 +301,28 @@ function hook_webform_component_delete($component) {
     ->execute();
 }
 
+/**
+ * Alter a Webform submission's header when exported.
+ */
+function hook_webform_csv_header_alter(&$header, $component) {
+  // Use the machine name for component headers, but only for the webform 
+  // with node 5 and components that are text fields.
+  if ($component['nid'] == 5 && $component['type'] == 'textfield') {
+    $header[2] = $component['form_key'];
+  }
+}
+
+/**
+ * Alter a Webform submission's data when exported.
+ */
+function hook_webform_csv_data_alter(&$data, $component, $submission) {
+  // If a value of a field was left blank, use the value from another
+  // field.
+  if ($component['cid'] == 1 && empty($data)) {
+    $data = $submission->data[2]['value'][0];
+  }
+}
+
 /**
  * Define components to Webform.
  *
@@ -320,9 +342,9 @@ function hook_webform_component_delete($component) {
  *     - spam_analysis
  *     - group
  *
- *   Note that most of these features do not indicate the default state, but 
+ *   Note that most of these features do not indicate the default state, but
  *   determine if the component can have this property at all. Setting
- *   "required" to TRUE does not mean that a component's fields will always be 
+ *   "required" to TRUE does not mean that a component's fields will always be
  *   required, but instead give the option to the administrator to choose the
  *   requiredness. See the example implementation for details on how these
  *   features may be set.
@@ -398,7 +420,7 @@ function hook_webform_component_info() {
       // Defaults to TRUE.
       'conditional' => TRUE,
 
-      // If this component allows other components to be grouped within it 
+      // If this component allows other components to be grouped within it
       // (like a fieldset or tabs). Defaults to FALSE.
       'group' => FALSE,
 
@@ -431,6 +453,97 @@ function hook_webform_component_info_alter(&$components) {
   $components['textarea']['label'] = t('Text box');
 }
 
+/**
+ * Alter the list of Webform component default values.
+ *
+ * @param $defaults
+ *   A list of component defaults as defined by _webform_defaults_COMPONENT().
+ * @param $type
+ *   The component type whose defaults are being provided.
+ *
+ * @see _webform_defaults_component()
+ */
+function hook_webform_component_defaults_alter(&$defaults, $type) {
+  // Alter a default for all component types.
+  $defaults['required'] = 1;
+
+  // Add a default for a new field added via hook_form_alter() or
+  // hook_form_FORM_ID_alter() for all component types.
+  $defaults['extra']['added_field'] = t('Added default value');
+
+  // Add or alter defaults for specific component types:
+  switch ($type) {
+    case 'select':
+      $defaults['extra']['optrand'] = 1;
+      break;
+
+    case 'textfield':
+    case 'textarea':
+      $defaults['extra']['another_added_field'] = t('Another added default value');
+  }
+}
+
+/**
+ * Alter access to a Webform submission.
+ *
+ * @param $node
+ *   The Webform node on which this submission was made.
+ * @param $submission
+ *   The Webform submission.
+ * @param $op
+ *   The operation to be performed on the submission. Possible values are:
+ *   - "view"
+ *   - "edit"
+ *   - "delete"
+ *   - "list"
+ * @param $account
+ *   A user account object.
+ * @return
+ *   TRUE if the current user has access to submission,
+ *   or FALSE otherwise.
+ */
+function hook_webform_submission_access($node, $submission, $op = 'view', $account = NULL) {
+  switch ($op) {
+    case 'view':
+      return TRUE;
+      break;
+    case 'edit':
+      return FALSE;
+      break;
+    case 'delete':
+      return TRUE;
+      break;
+    case 'list':
+      return TRUE;
+      break;
+  }
+}
+
+/**
+ * Determine if a user has access to see the results of a webform.
+ *
+ * Note in addition to the view access to the results granted here, the $account
+ * must also have view access to the Webform node in order to see results.
+ *
+ * @see webform_results_access().
+ *
+ * @param $node
+ *   The Webform node to check access on.
+ * @param $account
+ *   The user account to check access on.
+ * @return
+ *   TRUE or FALSE if the user can access the webform results.
+ */
+function hook_webform_results_access($node, $account) {
+  // Let editors view results of unpublished webforms.
+  if ($node->status == 0 && in_array('editor', $account->roles)) {
+    return TRUE;
+  }
+  else {
+    return FALSE;
+  }
+}
+
 /**
  * Return an array of files associated with the component.
  *
@@ -612,11 +725,11 @@ function _webform_display_component($component, $value, $format = 'html') {
 /**
  * A hook for changing the input values before saving to the database.
  *
- * Webform expects a component to consist of a single field, or a single array 
+ * Webform expects a component to consist of a single field, or a single array
  * of fields. If you have a component that requires a deeper form tree
- * you must flatten the data into a single array using this callback 
+ * you must flatten the data into a single array using this callback
  * or by setting #parents on each field to avoid data loss and/or unexpected
- * behavior. 
+ * behavior.
  *
  * Note that Webform will save the result of this function directly into the
  * database.
@@ -679,17 +792,21 @@ function _webform_help_component($section) {
 /**
  * Module specific instance of hook_theme().
  *
- * This allows each Webform component to add information into hook_theme().
+ * This allows each Webform component to add information into hook_theme(). If
+ * you specify a file to include, you must define the path to the module that
+ * this file belongs to.
  */
 function _webform_theme_component() {
   return array(
     'webform_grid' => array(
       'render element' => 'element',
       'file' => 'components/grid.inc',
+      'path' => drupal_get_path('module', 'webform'),
     ),
     'webform_display_grid' => array(
       'render element' => 'element',
       'file' => 'components/grid.inc',
+      'path' => drupal_get_path('module', 'webform'),
     ),
   );
 }
